using System;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{

	/**
	* BER TaggedObject - in ASN.1 nottation this is any object proceeded by
	* a [n] where n is some number - these are assume to follow the construction
	* rules (as with sequences).
	*/
	public class BERTaggedObject: DERTaggedObject
	{
		/**
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public BERTaggedObject(
			int             tagNo,
			DEREncodable    obj): base(tagNo, obj){}

		/**
		* @param flagExplicit true if an explicitly tagged object.
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public BERTaggedObject(
			bool         flagExplicit,
			int             tagNo,
			DEREncodable    obj): base(flagExplicit, tagNo, obj){}


		/**
		* create an implicitly tagged object that contains a zero
		* length sequence.
		*/
		public BERTaggedObject(
			int             tagNo): base(false, tagNo, new BERConstructedSequence()){}

		public override  void encode(DEROutputStream outStream)
			//throws IOException
		{
			if (typeof(ASN1OutputStream).IsInstanceOfType(outStream) ||
				typeof(BEROutputStream).IsInstanceOfType(outStream))
			{
				outStream.write(DERTags.CONSTRUCTED | DERTags.TAGGED | tagNo);
				outStream.write(0x80);

				if (!empty)
				{
					if (!flagExplicit)
					{
						if (typeof(ASN1OctetString).IsInstanceOfType(obj))
						{
							IEnumerator  e;

							if (typeof(BERConstructedOctetString).IsInstanceOfType(obj))
							{
								e = ((BERConstructedOctetString)obj).getObjects();
							}
							else
							{
								ASN1OctetString             octs = (ASN1OctetString)obj;
								BERConstructedOctetString   berO = new BERConstructedOctetString(octs.getOctets());

								e = berO.getObjects();
							}

							while (e.MoveNext())
							{
								outStream.writeObject(e.Current);
							}
						}
						else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
						{
							IEnumerator  e = ((ASN1Sequence)obj).getObjects();

							while (e.MoveNext())
							{
								outStream.writeObject(e.Current);
							}
						}
						else if (typeof(ASN1Set).IsInstanceOfType(obj))
						{
							IEnumerator  e = ((ASN1Set)obj).getObjects();

							while (e.MoveNext())
							{
								outStream.writeObject(e.Current);
							}
						}
						else
						{
							throw new Exception("not implemented: " + obj.GetType().ToString());
						}
					}
					else
					{
						outStream.writeObject(obj);
					}
				}

				outStream.write(0x00);
				outStream.write(0x00);
			}
			else
			{
				base.encode(outStream);
			}
		}
	}

}