using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{

	/**
	* DER IA5String object - this is an ascii string.
	*/
	public class DERIA5String: DERObject, DERString
	{
		String  m_string;

		/**
		* return a IA5 string from the passed in object
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERIA5String getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERIA5String).IsInstanceOfType(obj)))
			{
				return (DERIA5String)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERIA5String(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an IA5 String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERIA5String getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - with bytes.
		*/
		public DERIA5String(
			byte[]   in_string)
		{
			char[]  cs = new char[in_string.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				cs[i] = (char)(in_string[i] & 0xff);
			}

			this.m_string = new String(cs);
		}

		/**
		* basic constructor - with string.
		*/
		public DERIA5String(String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public byte[] getOctets()
		{
			char[]  cs = m_string.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = (byte)cs[i];
			}

			return bs; 
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.IA5_STRING, this.getOctets());
		}

		public override int GetHashCode()
		{
			return this.getString().GetHashCode();
		}

		public override bool Equals(Object  o)
		{
			if (!(typeof(DERIA5String).IsInstanceOfType(o)))
			{
				return false;
			}

			DERIA5String  s = (DERIA5String)o;

			return this.getString().Equals(s.getString());
		}
	}


}