using System;
using System.IO;
using System.Collections;
using org.bouncycastle.math;

namespace org.bouncycastle.asn1
{
	public class DERInteger: DERObject
	{
		byte[]      bytes;

		/**
		* return an integer from the passed in object
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERInteger getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERInteger).IsInstanceOfType(obj)))
			{
				return (DERInteger)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERInteger(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an Integer from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERInteger getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		public DERInteger(
			int         value)
		{
			bytes = BigInteger.valueOf(value).toByteArray();
		}

		public DERInteger(
			BigInteger   value)
		{
			bytes = value.toByteArray();
		}

		public DERInteger(
			byte[]   bytes)
		{
			this.bytes = bytes;
		}

		public BigInteger getValue()
		{
			return new BigInteger(bytes);
		}

		/**
		* in some cases positive values get crammed into a space,
		* that's not quite big enough...
		*/
		public BigInteger getPositiveValue()
		{
			return new BigInteger(1, bytes);
		}

		public override void encode(DEROutputStream outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.INTEGER, bytes);
		}
	    
		public override  bool Equals(Object  o)
		{
			if ((o == null) || !(typeof(DERInteger).IsInstanceOfType(o)))
			{
				return false;
			}

			DERInteger other = (DERInteger)o;

			if (bytes.Length != other.bytes.Length)
			{
				return false;
			}

			for (int i = 0; i != bytes.Length; i++)
			{
				if (bytes[i] != other.bytes[i])
				{
					return false;
				}
			}

			return true;
		}
		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}

}