using System;
using System.IO;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER TaggedObject - in ASN.1 nottation this is any object proceeded by
	* a [n] where n is some number - these are assume to follow the construction
	* rules (as with sequences).
	*/
	public class DERTaggedObject: ASN1TaggedObject
	{
		/**
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public DERTaggedObject(
			int             tagNo,
			DEREncodable    obj) : base(tagNo, obj){}

		/**
		* @param flagExplicit true if an explicitly tagged object.
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public DERTaggedObject(
			bool         flagExplicit,
			int             tagNo,
			DEREncodable    obj): base(flagExplicit, tagNo, obj) {}

		/**
		* create an implicitly tagged object that contains a zero
		* length sequence.
		*/
		public DERTaggedObject(
			int             tagNo): base(false, tagNo, new DERSequence()) {}

		public override void encode(DEROutputStream outStream)
			//throws IOException
		{
			if (!empty)
			{
				MemoryStream   bOut = new MemoryStream();
				DEROutputStream         dOut = new DEROutputStream(bOut);

				dOut.writeObject(obj);
				dOut.close();

				byte[]  bytes = bOut.ToArray();

				if (flagExplicit)
				{
					outStream.writeEncoded(DERTags.CONSTRUCTED | DERTags.TAGGED | tagNo, bytes);
				}
				else
				{
					//
					// need to mark constructed types...
					//
					if ((bytes[0] & DERTags.CONSTRUCTED) != 0)
					{
						bytes[0] = (byte)(DERTags.CONSTRUCTED | DERTags.TAGGED | tagNo);
					}
					else
					{
						bytes[0] = (byte)(DERTags.TAGGED | tagNo);
					}

					outStream.write(bytes);
				}
			}
			else
			{
				outStream.writeEncoded(DERTags.CONSTRUCTED | DERTags.TAGGED | tagNo, new byte[0]);
			}
		}
	}

}