using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* UTC time object.
	*/
	public class DERUTCTime: DERObject
	{
		String      time;

		/**
		* The correct format for this is YYMMDDHHMMSSZ (it used to be that seconds were
		* never encoded. When you're creating one of these objects from scratch, that's
		* what you want to use, otherwise we'll try to deal with whatever gets read from
		* the input stream... (this is why the input format is different from the getTime()
		* method output).
		* <p>
		*
		* @param time the time string.
		*/
		public DERUTCTime(
			String  time)
		{
			this.time = time;
		}

		internal DERUTCTime(
			byte[]  bytes)
		{
			//
			// explicitly convert to characters
			//
			char[]  dateC = new char[bytes.Length];

			for (int i = 0; i != dateC.Length; i++)
			{
				dateC[i] = (char)(bytes[i] & 0xff);
			}

			this.time = new String(dateC);
		}

		/**
		* return the time - always in the form of 
		*  YYMMDDhhmmssGMT(+hh:mm|-hh:mm).
		* <p>
		* Normally in a certificate we would expect "Z" rather than "GMT",
		* however adding the "GMT" means we can just use:
		* <pre>
		*     dateF = new SimpleDateFormat("yyMMddHHmmssz");
		* </pre>
		* To read in the time and get a date which is compatible with our local
		* time zone.
		*/
		public String getTime()
		{
			//
			// standardise the format.
			//
			if (time.Length == 11)
			{
				return time.Substring(0, 10) + "00GMT+00:00";
			}
			else if (time.Length == 13)
			{
				return time.Substring(0, 12) + "GMT+00:00";
			}
			else if (time.Length == 17)
			{
				return time.Substring(0, 12) + "GMT" + time.Substring(12, 3) + ":" + time.Substring(15, 2);
			}

			return time;
		}

		/**
		* return the time as an adjusted date with a 4 digit year. This goes
		* in the range of 1950 - 2049.
		*/
		public String getAdjustedTime()
		{
			String   d = this.getTime();

			if (d[0] < '5')
			{
				return "20" + d;
			}
			else
			{
				return "19" + d;
			}
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			char[]  cs = time.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			outStream.writeEncoded(DERTags.UTC_TIME, bs);
		}
	}

}