using System;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
	public class Attribute : DEREncodable
	{
		private DERObjectIdentifier attrType;
		private ASN1Set             attrValues;

		/**
			* return an Attribute object from the given object.
			*
			* @param o the object we want converted.
			* @exception ArgumentException if the object cannot be converted.
			*/
		public static Attribute getInstance(
			Object o)
		{
			if ((o == null) || (typeof(Attribute).IsInstanceOfType(o)))
			{
				return (Attribute)o;
			}
			if (typeof(ASN1Sequence).IsInstanceOfType(o))
			{
				return new Attribute((ASN1Sequence)o);
			}

			throw new ArgumentException("unknown object in factory");
		}
	
		public Attribute(
			ASN1Sequence seq)
		{
			attrType = (DERObjectIdentifier)seq.getObjectAt(0);
			attrValues = (ASN1Set)seq.getObjectAt(1);
		}

		public Attribute(
			DERObjectIdentifier attrType,
			ASN1Set             attrValues)
		{
			this.attrType = attrType;
			this.attrValues = attrValues;
		}

		public DERObjectIdentifier getAttrType()
		{
			return attrType;
		}
	
		public ASN1Set getAttrValues()
		{
			return attrValues;
		}

		/** 
		 * <pre>
		 * Attribute ::= SEQUENCE {
		 * 	attrType OBJECT IDENTIFIER,
		 * 	attrValues SET OF AttributeValue
		 * }
		 * </pre>
		 */
		public DERObject getDERObject()
		{
			ASN1EncodableVector v = new ASN1EncodableVector();

			v.add(attrType);
			v.add(attrValues);

			return new DERSequence(v);
		}
	}
}