
using System;
using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	*  extendedKeyUsage
	*  <pre>
	*      extendedKeyUsage ::= SEQUENCE SIZE (1..MAX) OF KeyPurposeId
	*  </pre>
	*/
	public class ExtendedKeyUsage: DEREncodable
	{
		Hashtable     usageTable = new Hashtable();
		ASN1Sequence  seq;

		public static ExtendedKeyUsage getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static ExtendedKeyUsage getInstance(
			Object obj)
		{
			if ((obj == null) || (typeof(ExtendedKeyUsage).IsInstanceOfType(obj)))
			{
				return (ExtendedKeyUsage)obj;
			}
	        
			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new ExtendedKeyUsage((ASN1Sequence)obj);
			}
	        
			throw new ArgumentException("Invalid ExtendedKeyUsage: " + obj.GetType().ToString());
		}

		public ExtendedKeyUsage(
			ASN1Sequence  seq)
		{
			this.seq = seq;

			IEnumerator e = seq.getObjects();

			while (e.MoveNext())
			{
				Object  o = e.Current;

				this.usageTable[o] = o;
			}
		}

		public ExtendedKeyUsage(
			ArrayList  usages)
		{
			ASN1EncodableVector v = new ASN1EncodableVector();
			IEnumerator         e = usages.GetEnumerator();

			while (e.MoveNext())
			{
				DERObject  o = (DERObject)e.Current;

				v.add(o);
				this.usageTable[o] = o;
			}

			this.seq = new DERSequence(v);
		}

		public bool hasKeyPurposeId(
			KeyPurposeId keyPurposeId)
		{
			return (usageTable[keyPurposeId] != null);
		}

		public DERObject getDERObject()
		{
			return seq;
		}
	}

}